@@ -47,7 +47,7 @@
 
 #define meta_XFree(p) do { if ((p)) XFree ((p)); } while (0)
 
-typedef struct MetaCompositor  MetaCompositor;
+typedef struct _MetaCompositor  MetaCompositor;
 typedef struct _MetaDisplay    MetaDisplay;
 typedef struct _MetaFrame      MetaFrame;
 typedef struct _MetaKeyBinding MetaKeyBinding;
@@ -159,6 +159,8 @@ struct _MetaDisplay
   Atom atom_net_wm_action_maximize_vert;
   Atom atom_net_wm_action_change_desktop;
   Atom atom_net_wm_action_close;
+  Atom atom_net_wm_action_above;
+  Atom atom_net_wm_action_below;
   Atom atom_net_wm_state_above;
   Atom atom_net_wm_state_below;
   Atom atom_net_startup_id;
@@ -254,7 +256,6 @@ struct _MetaDisplay
   MetaScreen *grab_screen;
   MetaWindow *grab_window;
   Window      grab_xwindow;
-  gulong      grab_start_serial;
   int         grab_button;
   int         grab_anchor_root_x;
   int         grab_anchor_root_y;
@@ -353,6 +354,14 @@ struct _MetaDisplay
   int render_event_base;
   int render_error_base;
 #endif
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+  int composite_event_base;
+  int composite_error_base;
+  int damage_event_base;
+  int damage_error_base;
+  int xfixes_event_base;
+  int xfixes_error_base;
+#endif
 #ifdef HAVE_XSYNC
   unsigned int have_xsync : 1;
 #define META_DISPLAY_HAS_XSYNC(display) ((display)->have_xsync)
@@ -371,6 +380,18 @@ struct _MetaDisplay
 #else
 #define META_DISPLAY_HAS_RENDER(display) FALSE
 #endif
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+  unsigned int have_composite : 1;
+  unsigned int have_damage : 1;
+  unsigned int have_xfixes : 1;
+#define META_DISPLAY_HAS_COMPOSITE(display) ((display)->have_composite)
+#define META_DISPLAY_HAS_DAMAGE(display) ((display)->have_damage)
+#define META_DISPLAY_HAS_XFIXES(display) ((display)->have_xfixes)
+#else
+#define META_DISPLAY_HAS_COMPOSITE(display) FALSE
+#define META_DISPLAY_HAS_DAMAGE(display) FALSE
+#define META_DISPLAY_HAS_XFIXES(display) FALSE
+#endif
 };
 
 /* Xserver time can wraparound, thus comparing two timestamps needs to take
@@ -415,6 +436,10 @@ void          meta_display_unmanage_windows_for_screen (MetaDisplay *display,
                                                         MetaScreen  *screen,
                                                         guint32      timestamp);
 
+/* Utility function to compare the stacking of two windows */
+int           meta_display_stack_cmp           (const void *a,
+                                                const void *b);
+
 /* A given MetaWindow may have various X windows that "belong"
  * to it, such as the frame window.
  */
@@ -450,7 +475,6 @@ gboolean meta_display_begin_grab_op (MetaDisplay *display,
                                      MetaGrabOp   op,
                                      gboolean     pointer_already_grabbed,
                                      gboolean     frame_action,
-                                     int          event_serial,
                                      int          button,
                                      gulong       modmask,
                                      guint32      timestamp,
