@@ -961,3 +961,34 @@ meta_ui_get_direction (void)
 
   return META_UI_DIRECTION_LTR;
 }
+
+GdkPixbuf *
+meta_ui_get_pixbuf_from_pixmap (Pixmap   pmap)
+{
+  GdkPixmap *gpmap;
+  GdkScreen *screen;
+  GdkPixbuf *pixbuf;
+  GdkColormap *cmap;
+  int width, height, depth;
+
+  gpmap = gdk_pixmap_foreign_new (pmap);
+  screen = gdk_drawable_get_screen (gpmap);
+
+  gdk_drawable_get_size (GDK_DRAWABLE (gpmap), &width, &height);
+  
+  depth = gdk_drawable_get_depth (GDK_DRAWABLE (gpmap));
+  if (depth <= 24)
+    cmap = gdk_screen_get_rgb_colormap (screen);
+  else
+    cmap = gdk_screen_get_rgba_colormap (screen);
+  
+  pixbuf = gdk_pixbuf_get_from_drawable (NULL, gpmap, cmap, 0, 0, 0, 0,
+                                         width, height);
+
+  g_object_unref (gpmap);
+
+  return pixbuf;
+}
+
+  
+  
